@@ -5,7 +5,21 @@ Created on 2011-05-12
 @author: berni
 '''
 
-import urllib
+try:
+    # for Python3
+    import urllib.parse as urllib
+except ImportError:
+    # for Python2
+    import urllib
+
+try:
+    unicode
+    DEFAULT_ENCODING = 'utf-8'
+except NameError:
+    # for Python3
+    uncode = str
+    DEFAULT_ENCODING = None
+
 
 
 def has_variable_name(s):
@@ -90,12 +104,12 @@ def parser_helper(key, val):
             if newkey is None:
                 raise MalformedQueryStringError
         newkey = int(newkey) if is_number(newkey) else newkey
-        val = int(val) if is_number(val) else val
+        if key == u'[]':  # val is the array key
+            val = int(val) if is_number(val) else val
         pdict[newkey] = val
     return pdict
 
-
-def parse(query_string, unquote=True, encoding='utf-8'):
+def parse(query_string, unquote=True, normalized=False, encoding=DEFAULT_ENCODING):
     '''
     Main parse function
     @param query_string:
@@ -103,6 +117,7 @@ def parse(query_string, unquote=True, encoding='utf-8'):
     @param encoding: An optional encoding used to decode the keys and values. Defaults to utf-8, which the W3C declares as a defaul in the W3C algorithm for encoding.
     @see http://www.w3.org/TR/html5/forms.html#application/x-www-form-urlencoded-encoding-algorithm
 
+    @param normalized: parse number key in dict to proper list ?
     '''
     mydict = {}
     plist = []
@@ -112,8 +127,8 @@ def parse(query_string, unquote=True, encoding='utf-8'):
         try:
             if unquote:
                 (var, val) = element.split("=")
-                var = urllib.unquote_plus(var)
-                val = urllib.unquote_plus(val)
+                var = urllib.unquote_plus(var.encode('ascii'))
+                val = urllib.unquote_plus(val.encode('ascii'))
             else:
                 (var, val) = element.split("=")
         except ValueError:
@@ -134,9 +149,42 @@ def parse(query_string, unquote=True, encoding='utf-8'):
             tempdict[k] = [tempdict[k], v]
         else:
             tempdict[k] = v
+
+    if normalized == True:
+        return _normalize(mydict)
     return mydict
 
 
+def _normalize(d):
+    '''
+    The above parse function generates output of list in dict form
+    i.e. {'abc' : {0: 'xyz', 1: 'pqr'}}. This function normalize it and turn
+    them into proper data type, i.e. {'abc': ['xyz', 'pqr']}
+
+    Note: if dict has element starts with 10, 11 etc.. this function won't fill
+    blanks.
+    for eg: {'abc': {10: 'xyz', 12: 'pqr'}} will convert to 
+    {'abc': ['xyz', 'pqr']}
+    '''
+    newd = {}
+    if isinstance(d, dict) == False:
+        return d
+    # if dictionary. iterate over each element and append to newd
+    for k, v in d.iteritems():
+        if type(v) is dict and type(v.keys()[0]) is int:
+            temp_new = []
+            for k1, v1 in v.items():
+                temp_new.append(_normalize(v1))
+            newd[k] = temp_new
+        elif type(v) is dict and v.keys()[0] == '':
+            newd[k] = v.values()[0]
+        elif type(v) is dict:
+            newd[k] = _normalize(v)
+        else:
+            newd[k] = v
+    return newd
+
+
 if __name__ == '__main__':
     """Compare speed with Django QueryDict"""
     from timeit import Timer
@@ -158,6 +206,6 @@ if __name__ == '__main__':
         t = Timer(statement, "from __main__ import parse")
         td = Timer(statementd, "from django import http")
         tqs = Timer(statementqs, "from urlparse import parse_qs")
-        print "Test string nr ".ljust(15), "querystring-parser".ljust(22), "Django QueryDict".ljust(22), "parse_qs"
-        print str(i).ljust(15), str(min(t.repeat(3, 10000))).ljust(22), str(min(td.repeat(3, 10000))).ljust(22), min(tqs.repeat(3, 10000))
+        print ("Test string nr ".ljust(15), "querystring-parser".ljust(22), "Django QueryDict".ljust(22), "parse_qs")
+        print (str(i).ljust(15), str(min(t.repeat(3, 10000))).ljust(22), str(min(td.repeat(3, 10000))).ljust(22), min(tqs.repeat(3, 10000)))
         i += 1
